/////////////////////////////////////////////////////////////////////////////
// Misc.cpp
/////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2000-2002 Grincheux  All rights reserved.
/////////////////////////////////////////////////////////////////////////////
//
//	This file is subject to the terms of the GNU General Public License as
//	published by the Free Software Foundation.  A copy of this license is
//	included with this software distribution in the file COPYING.  If you
//	do not have a copy, you may obtain a copy by writing to the Free
//	Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
//	This software is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details
/////////////////////////////////////////////////////////////////////////////
//
// This software was based on MultiDec 5.9 Those portions are
// Copyright (C) 1999/2000 Espresso (echter_espresso@hotmail.com)
//
/////////////////////////////////////////////////////////////////////////////
#include <windows.h>		// required for all Windows applications
#include <stdio.h>
#include <stdlib.h>

#include "general.h"
#include "../../../readwriteini/readwriteini.h"
#include "../../pluginvideo.h"



ConfigINI cfg[]={
//	parameter				type			variable

	{"Albator"				,CFG_SECTION	, NULL					},

	{"AUTO_PROCESS_VIDEO"	,CFG_BOOL		,&VideoPlugin.AutoProcessVideo},
	{"LIGNESTEST"			,CFG_INT		,&MasterTestzeilen		},
	{"LIGNESTEST1"			,CFG_INT		,&TZ[0]					},
	{"LIGNESTEST2"			,CFG_INT		,&TZ[1]					},
	{"LIGNESTEST3"			,CFG_INT		,&TZ[2]					},
	{"LIGNESTEST4"			,CFG_INT		,&TZ[3]					},
	{"LIGNESTEST5"			,CFG_INT		,&TZ[4]					},

	{"PAL_PLUS"				,CFG_BOOL		,&PalPlusActive },
	{"INITIAL_LOW"			,CFG_INT		,&InitialLow			},

	{"FILTRE_BLEU"			,CFG_BOOL		,&bFilterBlueLines		},
	{"FILTRE_SECAM"			,CFG_BOOL		,&bFilterSecam			},
	{"SEUIL_BLEU"			,CFG_INT		,&iUFilterThresholdPCent},
	{"SEUIL_ROUGE"			,CFG_INT		,&iVFilterThresholdPCent},
	{"SEUIL_VERT"			,CFG_INT		,&iYUVFilterThresholdPCent},
	{"SEUIL_NBPIX"			,CFG_INT		,&iPixFilterThreshold	},

	{NULL , NULL ,NULL}
};

char PluginINI[] = "Plugins/Albator.ini";


BOOL Load_INI()
{
	ChargerINI(cfg, PluginINI);
	return TRUE;
}


BOOL Write_INI()
{
	SauverINI(cfg, PluginINI);
	return TRUE;
}


extern __int64  CurrentLowValue;
// Convertit la luminosit en valeur 64 bits pour le plugin vido.
void SetLow(unsigned char bBrightness) 
{
	short Low;
	unsigned short *Lowptr;
	short nLevelLow; 
	short nLevelHigh;
	
	Lowptr=(unsigned short *)&CurrentLowValue;
	Low=InitialLow;  
	nLevelHigh=135;
	if ( bBrightness < 128 ) 
	{
		Low=Low+bBrightness;
		nLevelHigh=nLevelHigh+bBrightness;
		nLevelLow      =  45+bBrightness;
		
	}
	else
	{
		Low=Low-(256-bBrightness);
		nLevelHigh=nLevelHigh-(256-bBrightness);
		nLevelLow      =  45-(256-bBrightness);
	}
	
	if ( Low < 2 ) Low=0x02;
    if (nLevelHigh > 225) nLevelHigh=225;
	*Lowptr=Low;
    Lowptr++;
	*Lowptr=Low;
    Lowptr++;
	*Lowptr=Low;
    Lowptr++;
	*Lowptr=Low;
}


// Reconnaissance automatique du signal
//
// Si cette fonction a reconnu une information spcifique dans les donnes VBI ou Vido,
// il peut forcer K!TV  executer le plugin (en retournant TRUE).
//
// Notes:
// - Le mode AutoProcessVideo doit etre activ pour que cette fonctionnalit soit utilise.
//
// - Cette fonction est appele 25 fois par secondes (a chaque image capture).
//   
// - BYTE *pVBI est un pointeur vers les donnes VBI brutes
//   Si pVBI == NULL, alors il n'y a pas de donnes VBI.
//
// - Rien n'empeche d'utiliser le signal vido.
//

BOOL DetecteSignal(BYTE *pVBI)
{
	int i,j,row;
	BOOL text_lines_found=FALSE;
	static BOOL retval;
	unsigned char vbiline[2048];
	unsigned char vbibits[2048/4];
	unsigned char vbibytes[2048/4/8];
	int AutoID_Offset=4;
	unsigned int AutoID_sample_bits[48]=
	{
	 0,7,15,23,31,39,47,55,63,71,79,87,95,103,
	 111,119,126,134,142,150,158,166,174,182,
	 190,198,206,214,222,230,238,245,
	 253,261,269,277,285,293,301,309,317,325,
	 333,341,349,357,364,372
	};
	// Pas de signal VBI valide
	if (pVBI == NULL)
		return FALSE;
	
	text_lines_found=FALSE;
	
	for(row=0; row<32; row++)
	{
		for (i=0;i<2048;i++)
		{
       		if (*(pVBI++) < 95) vbiline[i]=0;
            else vbiline[i]=1;
		}
		i=0;
		while (vbiline[i]!=1) 
			if (i++ > 2048) break;
		if (i>200) continue;
		for (j=0;j<48 ;j++) vbibits[j]=vbiline[AutoID_sample_bits[j]+AutoID_Offset+i];

        for (i=0;i<48;i+=8)
		{
			vbibytes[i/8]=0;
			for (j=0;j<8;j++) vbibytes[i/8]|=vbibits[i+j]<<j;  // inverse order
		}
        if (vbibytes[0]==0x55) text_lines_found=TRUE;
		if ((vbibytes[0]==0x55) && (vbibytes[4]==0x15))
		{
			if  (vbibytes[5]&0x20)
			{
				retval = TRUE;
				break;
			}
			else retval = FALSE;
		}
	}
	if ( text_lines_found == FALSE ) retval=FALSE;

	return retval;
}


void ErreurFatale(char *Message)
{
	MessageBox(VideoPlugin.hwndParent,
		Message, "Plugin K!TV", MB_ICONSTOP|MB_OK);
}


/* ------------------------------------------------------------------------- */
__int64 GetRDTSCTime() 
/* ------------------------------------------------------------------------- */
{
    union {
	  __int64 ll;
	  struct { int lo;int hi ;} s;
	} time;

	__asm
	{
		cpuid
		rdtsc
		mov time.s.lo, eax
		mov time.s.hi, edx
	}

	return time.ll;
}

